/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef CONEXUSMMENDPOINTSTOPBUTTON_H
#define CONEXUSMMENDPOINTSTOPBUTTON_H

#include <map>
#include <gtkmm.h>

#include <conexus/endpoint.h>

namespace ConexusGtk
{

  /**
   * This widget can control the stop/start state of multiple Conexus endpoints.
   * To control a Conexus endpoint, just connect it and optionally set whether
   * the widget will control read stop/start, write stop/start or both.
   * 
   * @author Rick L Vinyard Jr
   */
  class EndpointStopButton : public Gtk::CheckButton
  {
    public:

      /**
       * @param use_text If true, the start/stop text label will be shown
       * @param use_image If true, the start/stop image will be shown
       */
      EndpointStopButton( bool show_text = true, bool show_image = true );

      /** Destructor */
      ~EndpointStopButton();

      /**
       * Place a Conexus Endpoint under the control of this widget
       * @param io The Conexus endpoint to control
       */
      void connect( Conexus::Endpoint& io );
      
      /**
       * Remove a Conexus endpoint from this widget's control
       */
      void disconnect( Conexus::Endpoint& io );

      /** The text to be displayed when the widget is in a started state */
      const Glib::ustring& start_text();
      
      /** The text to be displayed when the widget is in a stopped state */
      const Glib::ustring& stop_text();

      /** 
       * Set the start state text. Pango markup is used for display, so if desired
       * the text may also contain pango markup.
       */
      void set_start_text( const Glib::ustring& text );
      
      /** 
       * Reset the start text to the default value of "Connected  \<i\>\<small\>( Click to Disconnect )\</small\>\</i\>"
       */
      void set_start_text_to_default();

      /** 
       * Set the stop state text. Pango markup is used for display, so if desired
       * the text may also contain pango markup.
       */
      void set_stop_text( const Glib::ustring& text );
      
      /** 
       * Reset the stop text to the default value of "Disconnected  \<i\>\<small\>( Click to Connect )\</small\>\</i\>"
       */
      void set_stop_text_to_default();

      /** True if the text label will be shown, false if it will be hidden. */
      bool is_text_visible();
      
      /** Shows the text label */
      void show_text( );
      
      /** Hides the text label */
      void hide_text();

      /** The image to display when the widget is in a started state. */
      const Gtk::Image& start_image();
      
      /** The image to display when the widget is in a stopped state. */
      const Gtk::Image& stop_image();

      /** Set the start state image. */
      void set_start_image( Gtk::Image& image );
      
      /** Reset the start state image to the default stock connect icon. */
      void set_start_image_to_default();

      /** The image to display when the widge tis in a stopped state. */
      void set_stop_image( Gtk::Image& image );
      
      /** Reset the stopped state image to the default stock disconnect icon. */
      void set_stop_image_to_default();

      /** True if the start/stop image is shown; false if hidden. */
      bool is_image_visible();
      
      /** Shows the start/stop image */
      void show_image( );
      
      /** Hides the start/stop image */
      void hide_image( );

    protected:
      /** Holds the text and image widgets */
      Gtk::HBox m_hbox;
      
      /** Container for image */
      Gtk::EventBox m_imagebox;

      /** Widget displays the current stop/start text */
      Gtk::Label m_text;

      /** The text displayed when in a started state */
      Glib::ustring m_start_text;
      
      /** The text displayed when in a stopped state */
      Glib::ustring m_stop_text;
      
      /** The image displayed when in a started state */
      Gtk::Image* m_start_image;
      
      /** Default start image */
      Gtk::Image m_default_start_image;
      
      /** The image displayed when in a stopped state */
      Gtk::Image* m_stop_image;
      
      /** Default stop image */
      Gtk::Image m_default_stop_image;

      /** The type used for the set of endpoints this widget controls */
      typedef std::set<Conexus::Endpoint*> Controlled;
      
      /** The widget controls these endpoints */
      Controlled m_controlled;
      
      /** Callback to handle start/stop clicks */
      void on_button_toggled();

      /** Updates the display */
      void update_display( );

  };

}

#endif
